<?php
/**
 * Copyright 2007-2015 Horde LLC (http://www.horde.org/)
 *
 * @category   Horde
 * @package    Http
 * @subpackage UnitTests
 * @license    http://www.horde.org/licenses/bsd
 */

/**
 * @category   Horde
 * @package    Http
 * @subpackage UnitTests
 * @license    http://www.horde.org/licenses/bsd
 */
class Horde_Http_FopenTest extends Horde_Http_TestBase
{
    public function testBug12991()
    {
        $this->_skipMissingConfig();
        $client = new Horde_Http_Client(array(
            'request' => new Horde_Http_Request_Fopen()
        ));
        $response = $client->get($this->_server . '/doesntexist');
        $body = $response->getBody();
        $this->assertTrue(strlen($body) > 0);
    }
}
